# Generated by Django 2.2.5 on 2019-11-18 17:46

import django.db.models.deletion
from django.db import migrations, models

import human_lambdas.workflow_handler.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("user_handler", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Workflow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=128)),
                ("description", models.TextField()),
                ("inputs", human_lambdas.workflow_handler.fields.JSONField()),
                ("outputs", human_lambdas.workflow_handler.fields.JSONField()),
                ("created_at", models.DateTimeField(auto_now=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="user_handler.User",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="user_handler.Organization",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Task",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=128)),
                ("completed", models.DateTimeField(null=True)),
                ("input_data", human_lambdas.workflow_handler.fields.JSONField()),
                ("output_data", human_lambdas.workflow_handler.fields.JSONField()),
                (
                    "completed_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="user_handler.User",
                    ),
                ),
                (
                    "workflow",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="workflow_handler.Workflow",
                    ),
                ),
            ],
        ),
    ]



from django.contrib import admin

from .models import Task, TaskActivity, Workflow, WorkflowNotification

admin.site.register(Workflow)
admin.site.register(Task)
admin.site.register(TaskActivity)
admin.site.register(WorkflowNotification)



# Generated by Django 2.2.5 on 2019-11-19 18:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0002_auto_20191119_1511"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="email",
            field=models.EmailField(max_length=254, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['human_lambdas',
 'human_lambdas.data_handler',
 'human_lambdas.data_handler.tests',
 'human_lambdas.external',
 'human_lambdas.external.migrations',
 'human_lambdas.external.tests',
 'human_lambdas.hl_rest_api',
 'human_lambdas.metrics',
 'human_lambdas.metrics.tests',
 'human_lambdas.organization_handler',
 'human_lambdas.organization_handler.tests',
 'human_lambdas.templates_handler',
 'human_lambdas.templates_handler.tests',
 'human_lambdas.user_handler',
 'human_lambdas.user_handler.management.commands',
 'human_lambdas.user_handler.migrations',
 'human_lambdas.user_handler.tests',
 'human_lambdas.v1',
 'human_lambdas.workflow_handler',
 'human_lambdas.workflow_handler.management.commands',
 'human_lambdas.workflow_handler.migrations',
 'human_lambdas.workflow_handler.tests']

package_data = \
{'': ['*'],
 'human_lambdas.user_handler': ['templates/*'],
 'human_lambdas.workflow_handler.tests': ['data/*', 'data/encodings/*']}

install_requires = \
['Django==2.2.13',
 'PyJWT==1.7.1',
 'analytics-python==1.2.9',
 'cchardet==0.3.5',
 'click==7.1.2',
 'django-cors-headers==3.6.0',
 'django-next-prev>=1.1.0,<2.0.0',
 'django-rest-hooks-tmp==1.6.1',
 'djangorestframework-simplejwt==4.3.0',
 'djangorestframework==3.12.2',
 'drf-chunked-upload==0.4.2',
 'drf-yasg2>=1.19.4,<2.0.0',
 'email-validator==1.1.1',
 'google-auth==1.24.0',
 'google-cloud-storage==1.36.0',
 'gunicorn==20.0.04',
 'psycopg2-binary>=2.8.6,<3.0.0',
 'python-dotenv>=0.17.1,<0.18.0',
 'requests>=2.25.1,<3.0.0',
 'schema==0.7.2',
 'sendgrid==6.4.8',
 'sentry-sdk==0.19.5',
 'typing-extensions==3.7.4',
 'whitenoise>=5.2.0,<6.0.0']

entry_points = \
{'console_scripts': ['hl = human_lambdas.hl_cli:cli',
                     'human-lambdas = human_lambdas.hl_cli:cli']}

setup_kwargs = {
    'name': 'human-lambdas',
    'version': '1.0.47',
    'description': 'Open Source Human in the Loop platform for anyone to run their own private Mechanical Turk.',
    'long_description': None,
    'author': 'Human Lambdas Ltd.',
    'author_email': 'bernat@humanlambdas.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Human-Lambdas/human-lambdas',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.core.management.base import BaseCommand

from human_lambdas.user_handler.models import Organization, User


class Command(BaseCommand):
    help = "Updates the task count"

    def handle(self, *args, **options):
        users = User.objects.all()
        for user in users:
            org = Organization.objects.filter(user=user).first()
            if not org:
                self.stdout.write(
                    self.style.SUCCESS(
                        'No organization was found for user "%s"' % user.email
                    )
                )
            else:
                user.current_organization_id = org.pk
                user.save()
                self.stdout.write(
                    self.style.SUCCESS(
                        'Successfully updated current organization id for user "%s"'
                        % user.pk
                    )
                )



import rest_framework.authtoken.models
from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from human_lambdas.user_handler.models import Organization


@python_2_unicode_compatible
class Token(rest_framework.authtoken.models.Token):
    # key is no longer primary key, but still indexed and unique
    key = models.CharField(_("Key"), max_length=40, db_index=True, unique=True)
    # relation to user is a ForeignKey, so each user can have more than one token
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="auth_tokens",
        on_delete=models.CASCADE,
        verbose_name=_("User"),
    )
    organization = models.ForeignKey(
        Organization,
        related_name="auth_tokens",
        on_delete=models.CASCADE,
        verbose_name=_("Organization"),
    )

    class Meta:
        unique_together = (("user", "organization"),)



from django.core.management.base import BaseCommand

from human_lambdas.user_handler.models import Notification, User
from human_lambdas.workflow_handler.models import WorkflowNotification


class Command(BaseCommand):
    help = "Add notification object to each user"

    def handle(self, *args, **options):
        users = User.objects.all()
        for user in users:
            if not user.notifications:
                notification = Notification()
                notification.save()
                user.notifications = notification
                user.save()
            for org in user.organization_set.all():
                for workflow in org.workflow_set.all():
                    if not WorkflowNotification.objects.filter(
                        workflow=workflow, notification=user.notifications
                    ).exists():
                        WorkflowNotification(
                            workflow=workflow,
                            enabled=True,
                            notification=user.notifications,
                        ).save()
            self.stdout.write(
                self.style.SUCCESS(
                    "Set-up all workflow notifications for user %s" % user.pk
                )
            )



# Generated by Django 2.2.13 on 2020-11-18 11:09

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("user_handler", "0013_remove_notification_last_notified"),
    ]

    operations = [
        migrations.CreateModel(
            name="Token",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
                (
                    "key",
                    models.CharField(
                        db_index=True, max_length=40, unique=True, verbose_name="Key"
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="auth_tokens",
                        to="user_handler.Organization",
                        verbose_name="Organization",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="auth_tokens",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "unique_together": {("user", "organization")},
            },
        ),
    ]



# Generated by Django 2.2.5 on 2019-11-19 15:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="organization",
            name="user",
        ),
        migrations.AddField(
            model_name="organization",
            name="user",
            field=models.ManyToManyField(to="user_handler.User"),
        ),
    ]



import ast
import copy
import csv
from typing import Optional

from django.db.models import F
from django.http import HttpResponse

from human_lambdas.user_handler.notifications import send_notification
from human_lambdas.workflow_handler.models import Task, TaskActivity

from .data_transformation import ner_ext2int, ner_int2ext
from .data_validation import data_validation


# NER supports two mutually exclusive formats:
# 1) A column with the identifier with full support for the NER JSON format
# 2) A column with the identifier and the text sub-key as a short-hand to supply input text
def validate_keys_ner(title_row, winput):
    # Case 1: Validate key name
    value_count = title_row.count(winput["id"])
    # Case 2: Validate text subkey
    if f'{winput["id"]}.text' in title_row:
        value_count += 1
    # Raise exception if it has neither or both
    if value_count > 1:
        raise Exception("There are duplicate column names")


def validate_keys(title_row, workflow):
    for winput in workflow.inputs:
        if winput.get("type") == "named_entity_recognition":
            validate_keys_ner(title_row, winput)
        else:
            value_count = title_row.count(winput["id"])
            if value_count > 1:
                raise Exception("There are duplicate column names")
            if value_count == 0:
                raise Exception("The dataset is missing some columns")


# NER is a special case since it follows a different set of conventions
def extract_ner(data_item, row, title_row):
    # NER standard case
    if data_item["id"] in title_row:
        if title_row.index(data_item["id"]) < len(row):
            input_value = row[title_row.index(data_item["id"])]
            try:
                # We expect an object containing NER properties, so we will need the AST parser
                ner_object = (
                    ast.literal_eval(input_value)
                    if len(input_value) > 0
                    else {"text": ""}
                )
            except (ValueError, SyntaxError):
                raise Exception(
                    f"The value provided is not in right format: {input_value}"
                )
            # We should have an object matching the external API representation
            # We can leverage the API transformer
            ner_ext2int(data_item, {data_item["id"]: ner_object})
            return data_item

    # NER special case where the `text` sub-key is included
    if f'{data_item["id"]}.text' in title_row:
        if title_row.index(f'{data_item["id"]}.text') < len(row):
            input_value = row[title_row.index(f'{data_item["id"]}.text')]
            data_item[data_item["type"]]["value"] = input_value

    return data_item


# Extract Python literals in number, list, object based types
def extract_literals(data_item, row, title_row):
    if data_item["id"] in title_row:
        if title_row.index(data_item["id"]) < len(row):
            input_value = row[title_row.index(data_item["id"])].strip()
            try:
                data_item[data_item["type"]]["value"] = (
                    ast.literal_eval(input_value) if len(input_value) > 0 else None
                )
            except (ValueError, SyntaxError):
                raise Exception(
                    f"The value provided is not in right format: {input_value}"
                )
    return data_item


LITERAL_TYPES = [
    "text_sequence",
    "multiple_selection",
    "number",
    "form_sequence",
    "bounding_boxes",
    "binary",
]


def extract_default(data_item, row, title_row):
    if data_item["id"] in title_row:
        if title_row.index(data_item["id"]) < len(row):
            input_value = row[title_row.index(data_item["id"])]
            data_item[data_item["type"]]["value"] = input_value
    return data_item


def extract_value(w_data, row, title_row):
    data_item = copy.deepcopy(w_data)
    if "layout" in data_item:
        del data_item["layout"]
    if data_item["type"] in LITERAL_TYPES:
        return extract_literals(data_item, row, title_row)
    if data_item["type"] == "named_entity_recognition":
        return extract_ner(data_item, row, title_row)
    else:
        return extract_default(data_item, row, title_row)


def process_csv(
    csv_file, workflow, source, user, filename, region: Optional[str] = None
):
    dataset = csv.reader(csv_file)
    title_row = next(dataset)
    validate_keys(title_row, workflow)
    for row in dataset:
        data = [extract_value(w_input, row, title_row) for w_input in workflow.data]
        task = Task(
            data=data_validation(data),
            workflow=workflow,
            source=source,
            region=None if region in ["EU", None] else region,
        )
        task.save()
        TaskActivity(
            task=task,
            source="csv",
            filename=filename,
            created_by=user,
            action="created",
        ).save()
        workflow.n_tasks = F("n_tasks") + 1
        workflow.save()
    send_notification(workflow)


def extract_value_csv_export(task_data):
    if task_data["type"] == "named_entity_recognition":
        return ner_int2ext(task_data[task_data["type"]])
    else:
        return task_data[task_data["type"]]["value"]


def task_list_to_csv_response(task_list):
    workflow = task_list[0].workflow
    response = HttpResponse(content_type="text/csv")
    response[
        "Content-Disposition"
    ] = 'attachment; filename="workflow_{0}_completed_tasks.csv"'.format(workflow.id)
    writer = csv.writer(response)
    headers = [task_data["id"] for task_data in workflow.data]
    writer.writerow(headers)
    for task in task_list:
        writer.writerow(
            [
                next(
                    (
                        extract_value_csv_export(task_data)
                        for task_data in task.data
                        if task_data["id"] == workflow_data["id"]
                        and "value" in task_data[task_data["type"]]
                    ),
                    None,
                )
                for workflow_data in workflow.data
            ]
        )
    return response



# Generated by Django 2.2.5 on 2019-12-30 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0006_auto_20191219_1150"),
    ]

    operations = [
        migrations.AlterField(
            model_name="workflow",
            name="description",
            field=models.TextField(blank=True),
        ),
    ]



import copy
import os

from rest_framework import status
from rest_framework.test import APITestCase

from human_lambdas.user_handler.models import Organization
from human_lambdas.workflow_handler.models import Task
from human_lambdas.workflow_handler.tests import DATA_PATH
from human_lambdas.workflow_handler.tests.constants import (
    REGISTRATION_DATA,
    WORKFLOW_DATA_2,
)


class TestTaskList(APITestCase):
    def setUp(self):

        response = self.client.post("/v1/users/register", REGISTRATION_DATA)
        self.user_id = response.data["id"]
        self.org_id = Organization.objects.get(user__email="foo@bar.com").pk
        response = self.client.post(
            "/v1/users/token", {"email": "foo@bar.com", "password": "foowordbar"}
        )
        self.access_token = response.data["access"]

        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.get(
            "/v1/users/api-token",
        )
        self.token = response.data["token"]

        response = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            WORKFLOW_DATA_2,
            format="json",
        )
        self.workflow_id = response.data["id"]
        data_file = os.path.join(DATA_PATH, "dataset.csv")
        with open(data_file, encoding="ISO-8859-1") as f:
            data = {"file": f}
            response = self.client.post(
                "/v1/orgs/{0}/workflows/{1}/upload".format(
                    self.org_id, self.workflow_id
                ),
                data=data,
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.content)
        self.completed_tasks = 150
        for task in Task.objects.all()[: self.completed_tasks]:
            _ = self.client.post(
                "/v1/orgs/{0}/workflows/{1}/tasks/{2}/assign".format(
                    self.org_id, self.workflow_id, task.id
                ),
                data={"assigned_to": self.user_id},
                format="json",
            )
            data = copy.deepcopy(task.data)
            for idata in data:
                if idata["id"] == "foo":
                    idata["single_selection"]["value"] = "bar1"
            response_data = {"data": data}

            _ = self.client.patch(
                "/v1/orgs/{0}/workflows/{1}/tasks/{2}".format(
                    self.org_id, self.workflow_id, task.pk
                ),
                data=response_data,
                format="json",
            )

    def test_list_completed_task(self):
        self.client.credentials(HTTP_AUTHORIZATION="Token " + self.token)
        response = self.client.get(
            "/orgs/{}/workflows/{}/tasks/completed".format(
                self.org_id, self.workflow_id
            )
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(len(response.data["tasks"]), 100, response.data)
        response = self.client.get(
            "/orgs/{}/workflows/{}/tasks/completed".format(
                self.org_id, self.workflow_id
            ),
            format="json",
            data={"limit": 50},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(len(response.data["tasks"]), 50, response.data)
        for itask in response.data["tasks"]:
            self.assertEqual(itask["status"], "completed", itask)
        self.assertEqual(response.data["count"], self.completed_tasks, response.data)
        self.assertFalse("layout" in response.data["tasks"][0]["data"])
        self.assertEqual(type(response.data["tasks"][0]["data"]), dict)

    def test_existing_workflow(self):
        self.client.credentials(HTTP_AUTHORIZATION="Token " + self.token)
        response = self.client.get(
            "/orgs/{}/workflows/{}/tasks/completed".format(
                self.org_id, self.workflow_id
            )
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_non_existing_workflow(self):
        self.client.credentials(HTTP_AUTHORIZATION="Token " + self.token)
        response = self.client.get("/v1/orgs/10000/workflows/1000/completed")
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_hook_serializer(self):
        task = Task.objects.filter(status="completed").first()
        result = task.serialize_hook()
        self.assertEqual(result["id"], task.pk)
        self.assertEqual(type(result["data"]), dict)



import os

from rest_framework import status
from rest_framework.test import APITestCase

from human_lambdas.user_handler.models import Organization
from human_lambdas.workflow_handler.tests.constants import REGISTRATION_DATA

from . import DATA_PATH


class TestCSV(APITestCase):
    def setUp(self):
        self.file_path = os.path.join(DATA_PATH, "data_space.csv")
        self.total_rows = 3

        response = self.client.post("/v1/users/register", REGISTRATION_DATA)
        self.user_id = response.data["id"]

        self.org_id = Organization.objects.get(user__email="foo@bar.com").pk
        response = self.client.post(
            "/v1/users/token", {"email": "foo@bar.com", "password": "foowordbar"}
        )
        self.access_token = response.data["access"]
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        self.workflow_data = {
            "name": "uploader",
            "data": [
                {
                    "id": "car",
                    "name": "Car",
                    "type": "text",
                    "text": {"read_only": True},
                },
                {
                    "id": "img_1",
                    "name": "Image 1",
                    "type": "image",
                    "image": {"read_only": True},
                },
                {
                    "id": "img_2",
                    "name": "Image 2",
                    "type": "image",
                    "image": {"read_only": True},
                },
            ],
        }
        response = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            self.workflow_data,
            format="json",
        )
        self.workflow_id = response.data["id"]

    def test_uploading_csv(self):
        with open(self.file_path) as f:
            data = {"file": f}
            response = self.client.post(
                "/v1/orgs/{0}/workflows/{1}/upload".format(
                    self.org_id, self.workflow_id
                ),
                data=data,
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.content)



import logging
import os

from rest_framework import status
from rest_framework.test import APITestCase

from human_lambdas.user_handler.models import Notification, Organization, User
from human_lambdas.workflow_handler.models import Task, WebHook, Workflow
from human_lambdas.workflow_handler.tests.constants import (
    ALPHA,
    BETA,
    GAMMA,
    REGISTRATION_DATA,
    REGISTRATION_DATA_2,
    WORKFLOW_DATA,
    WORKFLOW_DATA_3,
)

logger = logging.getLogger(__name__)

_CURRENT_DIR = os.path.dirname(__file__)


class TestWebhook(APITestCase):
    def setUp(self):
        self.preset_user_name = "foo"
        self.preset_user_email = "foo@bar.com"
        self.organization_name = "fooinc"
        self.preset_user_password = "foowordbar"
        notification = Notification()
        notification.save()

        user = User(
            name=self.preset_user_name,
            email=self.preset_user_email,
            notifications=notification,
        )
        user.set_password(self.preset_user_password)
        user.save()
        organization = Organization(name=self.organization_name)
        organization.save()
        self.org_id = organization.id
        organization.add_admin(user)
        response = self.client.post(
            "/v1/users/token",
            {"email": "foo@bar.com", "password": self.preset_user_password},
        )
        self.admin_access_token = response.data["access"]
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        self.hook_url = "http://some.url.com"
        self.workflow_data = {
            "name": "foowf",
            "data": WORKFLOW_DATA["data"],
            "webhook": {"target": self.hook_url},
        }
        response = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            self.workflow_data,
            format="json",
        )
        self.workflow_id = response.data["id"]

    def test_admin_webhook_creation(self):
        self.assertIsNotNone(WebHook.objects.get(user__email="foo@bar.com"))

    def test_webhook_update(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        new_url = "http://someother.url.com"
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}".format(self.org_id, self.workflow_id),
            {"webhook": {"target": new_url}},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        hook = WebHook.objects.filter(workflow__id=response.data["id"]).first()
        self.assertIsNotNone(hook)
        self.assertEqual(hook.target, new_url)

    def test_webhook_retrieve(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        response = self.client.get(
            "/v1/orgs/{0}/workflows/{1}".format(self.org_id, self.workflow_id)
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        hook = WebHook.objects.filter(workflow__pk=self.workflow_id).first()
        self.assertIsNotNone(hook)
        self.assertEqual(hook.target, "http://some.url.com")

    def test_webhook_delete(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}".format(self.org_id, self.workflow_id),
            {"webhook": False},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(
            WebHook.objects.filter(workflow__pk=self.workflow_id).count(), 0
        )

    def test_webhook_endpoint(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        response = self.client.get(
            "/v1/orgs/{0}/workflows/{1}/webhook".format(self.org_id, self.workflow_id),
        )
        self.assertEqual(response.data["target"], self.hook_url)

        new_url = "http://someother.url.com"
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}/webhook".format(self.org_id, self.workflow_id),
            {"target": new_url},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(response.data["target"], new_url)
        self.assertEqual(
            WebHook.objects.get(workflow__pk=self.workflow_id).target, new_url
        )

    def test_invalid_url(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        self.workflow_data = {
            "name": "bar2",
            "inputs": [{"id": "foo", "name": "foo", "type": "text"}],
            "outputs": [
                {
                    "id": "foo",
                    "name": "foo",
                    "type": "single_selection",
                    "single_selection": {"options": ["foo1", "bar1"]},
                }
            ],
            "webhook": {"target": "not valid"},
        }
        response = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            self.workflow_data,
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["errors"][0]["field"], "webhook-->target")

    def test_webhook_delete_and_recreate(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}".format(self.org_id, self.workflow_id),
            {"webhook": False},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(
            WebHook.objects.filter(workflow__id=self.workflow_id).count(), 0
        )
        new_url = "http://someother.url.com"
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}".format(self.org_id, self.workflow_id),
            {"webhook": {"target": new_url}},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        hook = WebHook.objects.filter(workflow__pk=self.workflow_id).first()
        self.assertIsNotNone(hook)
        self.assertEqual(hook.target, new_url)


class TestWebhookTasks(APITestCase):
    def setUserClient(self, email):
        response = self.client.post(
            "/v1/users/token", {"email": email, "password": "foowordbar"}
        )
        self.access_token = response.data["access"]
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)

    def setUp(self):
        self.file_path = os.path.join(_CURRENT_DIR, "data", "test.csv")

        response = self.client.post("/v1/users/register", REGISTRATION_DATA)
        self.user_id = response.data["id"]

        _ = self.client.post("/v1/users/register", REGISTRATION_DATA_2)

        self.org_id = Organization.objects.get(user__email="foo@bar.com").pk
        response = self.client.post(
            "/v1/users/token", {"email": "foo@bar.com", "password": "foowordbar"}
        )
        self.access_token = response.data["access"]
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)

        workflow_data = {
            **WORKFLOW_DATA_3,
            **{
                "webhook": {"target": "https://en9sk43hft479.x.pipedream.net"},
            },
        }
        _ = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            workflow_data,
            format="json",
        )
        self.workflow_id = Workflow.objects.get(name="uploader").id
        with open(self.file_path) as f:
            data = {"file": f}
            response = self.client.post(
                "/v1/orgs/{0}/workflows/{1}/upload".format(
                    self.org_id, self.workflow_id
                ),
                data=data,
            )
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.content)
        workflow_data = {
            "name": "uploader2",
            "data": [
                ALPHA,
                BETA,
                GAMMA,
                {
                    "id": "foo",
                    "name": "foo",
                    "type": "single_selection",
                    "single_selection": {
                        "options": [
                            {"id": "foo2", "name": "foo2"},
                            {"id": "bar2", "name": "bar2"},
                        ],
                    },
                },
            ],
            "webhook": {"target": "https://en9sk43hft479.x.pipedream.net"},
        }
        _ = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            workflow_data,
            format="json",
        )

    def test_complete_single_selection_task(self):
        workflow = Workflow.objects.get(id=self.workflow_id)
        tasks = Task.objects.filter(workflow=workflow).all()
        output_values = ["foo1", "bar1"]
        for output_value, task in zip(output_values, tasks):
            data = task.data

            _ = self.client.post(
                "/v1/orgs/{0}/workflows/{1}/tasks/{2}/assign".format(
                    self.org_id, self.workflow_id, task.id
                ),
                data={"assigned_to": self.user_id},
                format="json",
            )

            for idata in data:
                if idata["id"] == "foo":
                    idata[idata["type"]]["value"] = output_value
            data_payload = {"data": data, "assigned_to": self.user_id}
            response = self.client.patch(
                "/v1/orgs/{0}/workflows/{1}/tasks/{2}".format(
                    self.org_id, self.workflow_id, task.id
                ),
                data=data_payload,
                format="json",
            )

            self.assertEqual(response.status_code, status.HTTP_200_OK, response.content)
            self.assertEqual("completed", response.data["status"])
            self.assertEqual(data, response.data["data"])
            self.assertEqual("completed", Task.objects.get(id=task.id).status)

    def test_find_and_fire(self):
        workflow = Workflow.objects.get(pk=self.workflow_id)
        filters = {
            "event": "task.completed",
            "workflow": workflow,
        }
        hooks = WebHook.objects.filter(**filters)
        self.assertEqual(len(hooks), 1)


class TestWebhookEndpoint(APITestCase):
    def setUp(self):
        self.preset_user_name = "foo"
        self.preset_user_email = "foo@bar.com"
        self.organization_name = "fooinc"
        self.preset_user_password = "foowordbar"
        notification = Notification()
        notification.save()

        user = User(
            name=self.preset_user_name,
            email=self.preset_user_email,
            notifications=notification,
        )
        user.set_password(self.preset_user_password)
        user.save()
        organization = Organization(name=self.organization_name)
        organization.save()
        self.org_id = organization.id
        organization.add_admin(user)
        response = self.client.post(
            "/v1/users/token",
            {"email": "foo@bar.com", "password": self.preset_user_password},
        )
        self.admin_access_token = response.data["access"]
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        self.workflow_data = WORKFLOW_DATA
        response = self.client.post(
            "/v1/orgs/{}/workflows/create".format(self.org_id),
            self.workflow_data,
            format="json",
        )
        self.workflow_id = response.data["id"]

    def test_webhook_endpoint(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.admin_access_token)
        response = self.client.get(
            "/v1/orgs/{0}/workflows/{1}/webhook".format(self.org_id, self.workflow_id),
        )
        self.assertEqual(response.data["target"], "")

        new_url = "http://some.url.com"
        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}/webhook".format(self.org_id, self.workflow_id),
            {"target": new_url},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)
        self.assertEqual(response.data["target"], new_url)
        self.assertEqual(
            WebHook.objects.get(workflow__pk=self.workflow_id).target, new_url
        )

        response = self.client.patch(
            "/v1/orgs/{0}/workflows/{1}/webhook".format(self.org_id, self.workflow_id),
            {"target": ""},
            format="json",
        )
        self.assertEqual(
            response.status_code, status.HTTP_204_NO_CONTENT, response.data
        )
        self.assertFalse(WebHook.objects.filter(workflow__pk=self.workflow_id).exists())



import logging

from django.test.utils import override_settings
from rest_framework import status
from rest_framework.test import APITestCase

from human_lambdas.user_handler.models import Organization, User

logger = logging.getLogger(__name__)
logging.disable(logging.DEBUG)


class TestInvite(APITestCase):
    def setUp(self):
        self.preset_user_name = "foo"
        self.preset_user_email = "foo@bar.com"
        self.preset_changed_email = "bar@foo.com"
        self.organization_name = "barinc"
        self.preset_user_password = "foowordbar"

        user = User(name=self.preset_user_name, email=self.preset_user_email)
        user.set_password(self.preset_user_password)
        user.save()

        response = self.client.post(
            "/v1/users/token", {"email": "foo@bar.com", "password": "foowordbar"}
        )
        self.access_token = response.data["access"]
        self.refresh = response.data["refresh"]

        organization = Organization(name=self.organization_name)
        organization.save()
        self.org_id = organization.id
        organization.add_admin(user)

    @override_settings(DEBUG=True)
    def test_endpoint_call(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "seanfahy45@gmail.com,alpha@beta.com,gamma@delta.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(DEBUG=True)
    def test_endpoint_call_with_spaces(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lambda@sigma.com     ,alpha@beta.com,    gamma@delta.com    "},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(DEBUG=True)
    def test_endpoint_call_duplicate_emails(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {
                "emails": "lambda@sigma.com,alpha@beta.com,gamma@delta.com,alpha@beta.com",
            },
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(DEBUG=True)
    def test_endpoint_call_no_jwt(self):
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lambda@sigma.com,alpha@beta.com,gamma@delta.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    @override_settings(DEBUG=True)
    def test_endpoint_call_already_added_email(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lambda@sigma.com,foo@bar.com,gamma@delta.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(DEBUG=True)
    def test_endpoint_call_invalid_emails(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lambda@sigma.com,foo@foo,bar.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(DEBUG=True)
    def test_endpoint_call_invalid_emails_and_already_added_emails(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lambda@sigma.com,foo@foo,bar.com,foo@bar.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(DEBUG=True)
    def test_endpoint_call_email_with_underscore(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "lam__bda@sigma.com,foo@foobarinc.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_endpoint_call_get(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.get("/v1/orgs/{0}/invite".format(self.org_id))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_endpoint_call_delete(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "delete@me.com"},
        )
        response = self.client.delete(
            "/v1/orgs/{0}/invite".format(self.org_id), {"email": "delete@me.com"}
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_endpoint_call_patch(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.post(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"emails": "patch@me.com"},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.client.patch(
            "/v1/orgs/{0}/invite".format(self.org_id),
            {"email": "patch@me.com", "admin": True},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.client.get("/v1/orgs/{0}/invite".format(self.org_id))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_endpoint_call_delete_no_invite(self):
        self.client.credentials(HTTP_AUTHORIZATION="Bearer " + self.access_token)
        response = self.client.delete(
            "/v1/orgs/{0}/invite".format(self.org_id), {"email": "delete@me.com"}
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



# Generated by Django 2.2.5 on 2020-03-09 17:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0014_auto_20200303_0949"),
    ]

    operations = [
        migrations.RenameField(
            model_name="task",
            old_name="completed_by",
            new_name="assigned_to",
        ),
        migrations.AddField(
            model_name="task",
            name="assigned_at",
            field=models.DateTimeField(null=True),
        ),
    ]



"""
WSGI config for hl_rest_api project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/howto/deployment/wsgi/
"""

import os
from pathlib import Path

from django.core.wsgi import get_wsgi_application

if Path(".env").exists():
    from dotenv import load_dotenv

    load_dotenv(".env")

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "human_lambdas.hl_rest_api.settings")

application = get_wsgi_application()
